@@ -1,5 +1,5 @@
 package Bot::Cobalt::DB;
-$Bot::Cobalt::DB::VERSION = '0.018001';
+$Bot::Cobalt::DB::VERSION = '0.019001';
 ## Uses proper retie-after-lock technique for locking
 
 use v5.10;
@@ -316,6 +316,7 @@ sub dbdump {
 
   ## shallow copy to drop tied()
   my %copy = %{ $self->tied };
+  return \%copy if lc($format) eq 'hash';
 
   my $dumper = Bot::Cobalt::Serializer->new( Format => $format );
 
@@ -481,6 +482,8 @@ of keys in the database in scalar context.
 
 You can serialize/export the entirety of the DB via B<dbdump>.
 
+  ## Export to a HASH
+  my $dbcopy = $db->dbdump('HASH');
   ## YAML::Syck
   my $yamlified = $db->dbdump('YAML');
   ## YAML::XS
