@@ -1,5 +1,5 @@
 package Bot::Cobalt::Utils;
-$Bot::Cobalt::Utils::VERSION = '0.018001';
+$Bot::Cobalt::Utils::VERSION = '0.019001';
 use strictures 2;
 use Carp;
 use Scalar::Util 'reftype';
@@ -284,8 +284,10 @@ sub _time_breakdown ($) {
 
   my $days   = int $diff / 86400;
   my $sec    = $diff % 86400;
-  my $hours  = int $sec / 3600;  $sec   %= 3600;
-  my $mins   = int $sec / 60;    $sec   %= 60;
+  my $hours  = int $sec / 3600;
+  $sec %= 3600;
+  my $mins   = int $sec / 60;
+  $sec %= 60;
 
   ($days, $hours, $mins, $sec)
 }
@@ -309,19 +311,22 @@ sub secs_to_str ($) {
   my ($diff) = @_;
   return unless defined $diff;
   my ($days, $hours, $mins, $sec) = _time_breakdown($diff);
-  sprintf "%d days, %2.2d:%2.2d:%2.2d", $days, $hours, $mins, $sec
+  my $plural = $days == 1 ? 'day' : 'days';
+  sprintf "%d $plural, %2.2d:%2.2d:%2.2d", $days, $hours, $mins, $sec
 }
 
 sub secs_to_str_y {
   my ($diff) = @_;
   return unless defined $diff;
   my ($days, $hrs, $mins, $sec) = _time_breakdown($diff);
-  my $yrs = int $days / 365;   $days %= 365;
-  my $plural = $yrs > 1 ? 'years' : 'year';
+  my $yrs = int $days / 365;
+  $days %= 365;
+  my $plural_y = $yrs > 1 ? 'years' : 'year';
+  my $plural_d = $days == 1 ? 'day' : 'days';
   $yrs ?
-    sprintf "%d $plural, %d days, %2.2d:%2.2d:%2.2d",
+    sprintf "%d $plural_y, %d $plural_d, %2.2d:%2.2d:%2.2d",
       $yrs, $days, $hrs, $mins, $sec
-    : sprintf "%d days, %2.2d:%2.2d:%2.2d",
+    : sprintf "%d $plural_d, %2.2d:%2.2d:%2.2d",
         $days, $hrs, $mins, $sec
 }
 
@@ -434,12 +439,12 @@ Useful for uptime reporting, for example:
   my $delta = time() - $your_start_TS;
   my $uptime_str = secs_to_str $delta;
 
-Returns time formatted as: C<< <D> days, <H>:<M>:<S> >>
+Returns time formatted as: C<< <D> day(s), <H>:<M>:<S> >>
 
 =head3 secs_to_str_y
 
 Like L</secs_to_str>, but includes year calculation and returns time formatted
-as: C<< <Y> years, <D> days, <H>:<M>:<S> >> B<if> there are more than 365
+as: C<< <Y> year(s), <D> day(s), <H>:<M>:<S> >> B<if> there are more than 365
 days; otherwise the same format as L</secs_to_str> is returned.
 
 (Added in C<v0.18.1>)
